# -*- coding: utf-8 -*- 
# @Time    : 03/21/2024 
# @Author  : nzooherd 
# @File    : log.py 
# @Software: Visual Studio Code

import asyncio
import logging
import logging.handlers
import os
import sys
from functools import partialmethod, partial
import telegram

from config import Config


logging.EVENT = 100
logging.addLevelName(logging.EVENT, 'EVENT')
logging.Logger.event = partialmethod(logging.Logger.log, logging.EVENT)
logging.event = partial(logging.log, logging.EVENT)

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

LOG_FORMAT = ('%(levelname)s - %(asctime)s - process: %(process)d - %(filename)s - %(name)s - %(lineno)d - %(module)s '
              '- %(message)s')
EVENT_FORMAT = '%(message)s'

class _AnsiColorizer:
    """
    A colorizer is an object that loosely wraps around a stream, allowing
    callers to write text to the stream in a particular color.

    Colorizer classes must implement C{supported()} and C{write(text, color)}.
    """
    _colors = dict(black=30, red=31, green=32, yellow=33,
                   blue=34, magenta=35, cyan=36, white=37)

    def __init__(self, stream):
        self.stream = stream

    @classmethod
    def supported(cls, stream=sys.stdout):
        """
        A class method that returns True if the current platform supports
        coloring terminal output using this method. Returns False otherwise.
        """
        if not stream.isatty():
            return False  # auto color only on TTYs
        try:
            import curses
        except ImportError:
            return False
        else:
            try:
                try:
                    return curses.tigetnum("colors") > 2
                except curses.error:
                    curses.setupterm()
                    return curses.tigetnum("colors") > 2
            except:
                raise
                # guess false in case of error
                return False

    def write(self, text, color):
        """
        Write the given text to the stream in the given color.

        @param text: Text to be written to the stream.

        @param color: A string label for a color. e.g. 'red', 'white'.
        """
        color = self._colors[color]
        self.stream.write(f'\033[{color}m{text}\033[0m\n')


class ColorHandler(logging.StreamHandler):
    def __init__(self, stream=sys.stdout):
        super(ColorHandler, self).__init__(_AnsiColorizer(stream))

    def emit(self, record):
        msg_colors = {
            logging.DEBUG: "green",
            logging.INFO: "blue",
            logging.WARNING: "yellow",
            logging.ERROR: "red"
        }

        color = msg_colors.get(record.levelno, "blue")
        self.stream.write(self.formatter.format(record), color)


class TelegramHandler(logging.Handler):

    def __init__(self) -> None:
        super().__init__(level=logging.EVENT)
        self.bot = telegram.Bot("7307639396:AAGl4oTmGgjezvVi9GX9htytePRhmI_o-UU")
        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)

    def send_message(self, message: str):
        try:
            send_coroutine = self.bot.send_message(chat_id=Config.TELEGRAM_ADMIN_USER_ID[0], text=message)
            if self.loop.is_running():
                self.loop.create_task(send_coroutine)
            else:
                self.loop.run_until_complete(send_coroutine)
        except Exception as e:
            logger.error(f"Error sending message to telegram: {e}")

    def emit(self, record: logging.LogRecord):
        if not Config.LOG_SEND_TO_TELEGRAM:
            return
        self.send_message(record.message)


def config_logger(app_name: str):
    log_formatter = logging.Formatter(LOG_FORMAT)
    event_formatter = logging.Formatter(EVENT_FORMAT)

    color_handler = ColorHandler()
    color_handler.setLevel(Config.LOG_LEVEL)
    color_handler.setFormatter(log_formatter)

    # 创建一个处理器用于 Error 和 Critical 级别的日志
    error_log_file = os.path.join(Config.ROOT_DIR, "logs/error", f"{app_name}.log")
    error_file_handler = logging.handlers.TimedRotatingFileHandler(
        filename=error_log_file, 
        when="midnight", 
        backupCount=30
    )
    error_file_handler.setLevel(logging.ERROR)
    error_file_handler.setFormatter(log_formatter)

    # 创建一个处理器用于其他所有级别的日志
    general_log_file = os.path.join(Config.ROOT_DIR, "logs/info", f"{app_name}.log")
    general_file_handler = logging.handlers.TimedRotatingFileHandler(
        filename=general_log_file, 
        when="midnight", 
        backupCount=30
    )
    general_file_handler.setLevel(logging.DEBUG)
    general_file_handler.setFormatter(log_formatter)

    telegram_handler = TelegramHandler()
    telegram_handler.setLevel(logging.EVENT)
    telegram_handler.setFormatter(event_formatter)

    logger.addHandler(color_handler)
    logger.addHandler(error_file_handler)
    logger.addHandler(general_file_handler)
    logger.addHandler(telegram_handler)
